@@ -75,8 +75,6 @@ enum {
 	LAST_SIGNAL
 };
 
-#define	RESET_BACKGROUND_MAGIC_IMAGE_NAME "reset.png"
-
 static guint signals[LAST_SIGNAL];
 
 struct EelBackgroundDetails {
@@ -922,18 +920,13 @@ void
 eel_background_receive_dropped_background_image (EelBackground *background,
 						      const char *image_uri)
 {
-	/* Special case the reset-background image by file name. */
-	if (eel_str_has_suffix (image_uri, "/" RESET_BACKGROUND_MAGIC_IMAGE_NAME)) {
-		eel_background_reset (background);
-	} else {
-		/* Currently, we only support tiled images. So we set the placement.
-		 * We rely on eel_background_set_image_uri_and_color to emit
-		 * the SETTINGS_CHANGED & APPEARANCE_CHANGE signals.
-		 */
-		eel_background_set_image_placement_no_emit (background, EEL_BACKGROUND_TILED);
+	/* Currently, we only support tiled images. So we set the placement.
+	 * We rely on eel_background_set_image_uri_and_color to emit
+	 * the SETTINGS_CHANGED & APPEARANCE_CHANGE signals.
+	 */
+	eel_background_set_image_placement_no_emit (background, EEL_BACKGROUND_TILED);
 
-		eel_background_set_image_uri_and_color (background, image_uri, NULL);
-	}
+	eel_background_set_image_uri_and_color (background, image_uri, NULL);
 }
 
 static GtkStyleClass *
@@ -1219,35 +1212,42 @@ eel_background_is_too_complex_for_gtk_style (EelBackground *background)
 gboolean
 eel_background_is_dark (EelBackground *background)
 {
-	GdkColor color;
-	int intensity, intensity2;
+	GdkColor color, end_color;
+	int intensity;
 	char *start_color_spec, *end_color_spec;
+	guint32 argb;
+	guchar a;
 
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
 	
-	if (background->details->image != NULL) {
-		eel_gdk_pixbuf_average_value (background->details->image, &color);
-	} else if (background->details->is_solid_color) {
+	if (background->details->is_solid_color) {
 		eel_gdk_color_parse_with_white_default (background->details->color, &color);	
 	} else {
 		start_color_spec = eel_gradient_get_start_color_spec (background->details->color);
-		end_color_spec = eel_gradient_get_end_color_spec (background->details->color);
-		
 		eel_gdk_color_parse_with_white_default (start_color_spec, &color);	
-		intensity = (((color.red >> 8) * 77) + ((color.green >> 8) * 150) + ((color.blue >> 8) * 28)) >> 8;		 
-		eel_gdk_color_parse_with_white_default (end_color_spec, &color);	
-		intensity2 = (((color.red >> 8) * 77) + ((color.green >> 8) * 150) + ((color.blue >> 8) * 28)) >> 8;		 
-		
 		g_free (start_color_spec);
+
+		end_color_spec = eel_gradient_get_end_color_spec (background->details->color);
+		eel_gdk_color_parse_with_white_default (end_color_spec, &end_color);
 		g_free (end_color_spec);
 		
-		return (intensity + intensity2) < 320;
+		color.red = (color.red + end_color.red) / 2;
+		color.green = (color.green + end_color.green) / 2;
+		color.blue = (color.blue + end_color.blue) / 2;
+	}
+	if (background->details->image != NULL) {
+		argb = eel_gdk_pixbuf_average_value_argb (background->details->image);
+		
+		a = argb >> 24;
+
+		color.red = (color.red * (0xFF - a) + ((argb >> 16) & 0xFF) * 0x101 * a) / 0xFF;
+		color.green = (color.green * (0xFF - a) + ((argb >> 8) & 0xFF) * 0x101 * a) / 0xFF;
+		color.blue = (color.blue * (0xFF - a) + (argb & 0xFF) * 0x101 * a) / 0xFF;
 	}
 	
-	intensity = (((color.red >> 8) * 77) + ((color.green >> 8) * 150) + ((color.blue >> 8) * 28)) >> 8;		 
+	intensity = (color.red * 77 + color.green * 150 + color.blue * 28) >> 16;		 
 	return intensity < 160; /* biased slightly to be dark */
 }
-
    
 /* handle dropped colors */
 void
