@@ -157,7 +157,7 @@ eel_enumeration_get_nth_description_translated (const EelEnumeration *enumeratio
 	untranslated_description = eel_string_list_nth (enumeration->descriptions, n);
 	g_return_val_if_fail (untranslated_description != NULL, NULL);
 
-	translated_description = _(untranslated_description);
+	translated_description = gettext (untranslated_description);
 
  	/* If not translation is found, return untranslated property as-is. */
  	if (translated_description == untranslated_description) {
