@@ -694,7 +694,8 @@ collect_fonts_from_directory (const char *font_directory,
 	while (TRUE) {
 		info = gnome_vfs_file_info_new ();
 		result = gnome_vfs_directory_read_next (directory, info);
-		if (result == GNOME_VFS_OK) {
+		if (result == GNOME_VFS_OK
+		    && (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE) != 0) {
 			if (strcasecmp (info->mime_type, POSTSCRIPT_FONT_MIME_TYPE) == 0) {
 				*postscript_font_list = g_list_prepend (*postscript_font_list,
 					g_strconcat (font_directory,
@@ -1346,7 +1347,6 @@ eel_font_manager_get_bold (const char *plain_font)
 	FindData data;
 
 	g_return_val_if_fail (eel_strlen (plain_font) > 0, NULL);
-	g_return_val_if_fail (eel_font_manager_file_is_scalable_font (plain_font), NULL);
 
 	ensure_local_font_table ();
 
@@ -1398,7 +1398,6 @@ eel_self_check_font_manager (void)
 	FontDescriptionTable *table;
 	const FontDescription *description;
 	GList *font_table_list = NULL;
-	char *original_current_dir;
 
 	/* chop_off_comments() */
 	EEL_CHECK_STRING_RESULT (call_chop_off_comments ("foo bar"), "foo bar");
@@ -1409,8 +1408,6 @@ eel_self_check_font_manager (void)
 	EEL_CHECK_STRING_RESULT (call_chop_off_comments ("\\#foo bar"), "\\#foo bar");
 	EEL_CHECK_STRING_RESULT (call_chop_off_comments ("\\##foo bar"), "\\#");
 
-	original_current_dir = g_get_current_dir ();
-
 	g_return_if_fail (g_file_exists (SOURCE_FONT_DIRECTORY));
 
 	g_return_if_fail (g_file_exists (TEST_FONT1));
