@@ -1974,7 +1974,7 @@ eel_gtk_class_name_make_like_existing_type (const char *class_name,
 		gtk_rc_add_widget_class_style (style->rc_style, class_name);
 	}
 		
-	gtk_widget_destroy (temporary);
+	gtk_object_sink (GTK_OBJECT (temporary));
 }
 
 /* helper function for eel_get_window_list_ordered_front_to_back () */
@@ -2163,7 +2163,7 @@ eel_gtk_get_system_font (void)
 	font = label->style->font;
 	gdk_font_ref (font);
 
- 	gtk_widget_destroy (label);
+ 	gtk_object_sink (GTK_OBJECT (label));
 
 	return font;
 }
